/* Main function for root server.
   Copyright (C) 2003 Free Software Foundation, Inc.
   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <l4.h>

#include "printf.h"

int
main (void)
{
  l4_api_version_t version;
  l4_api_flags_t flags;
  l4_kip_t kip = l4_kernel_interface (&version, &flags, 0);
  l4_kern_desc_t kern;
  char *features;
  l4_proc_desc_t proc;
  int nr;
  l4_memory_desc_t mem;

  cls ();

  printf ("Kernel Interface Page: 0x%x/%u (%c%c%c%c)\n", (unsigned long) kip,
	  kip->kip_area_info.log2_size,
	  kip->magic[0], kip->magic[1], kip->magic[2], kip->magic[3]);
  printf ("API Version: ");
  if (version.version == 0x02)
    printf ("Version 2");
  else if (version.version == 0x83 && version.subversion == 0x80)
    printf ("Experimental Version X.0");
  else if (version.version == 0x83 && version.subversion == 0x81)
    printf ("Experimental Version X.1");
  else if (version.version == 0x84)
    printf ("Experimental Version X.2 rev %u", version.subversion);
  else if (version.version == 0x04)
    printf ("Version 4 rev %u", version.subversion);
  else if (version.version & 0x80)
    printf ("Unknown Experimental Version");
  else
    printf ("Unknown Version");
  printf (" (0x%x, 0x%x)\n", version.version, version.subversion);
  printf ("API Flags: ");
  switch (flags.endian)
    {
    case L4_API_FLAGS_LITTLE_ENDIAN:
      printf ("little endian, ");
      break;

    case L4_API_FLAGS_BIG_ENDIAN:
      printf ("big endian, ");
      break;

    default:
      printf ("unknown endian, ");
      break;
    }
  switch (flags.endian)
    {
    case L4_API_FLAGS_WORDSIZE_32:
      printf ("32-bit\n");
      break;

    case L4_API_FLAGS_WORDSIZE_64:
      printf ("64-bit\n");
      break;

    default:
      printf ("unknown wordsize\n");
      break;
    }

  printf ("Kernel: ");
  kern = (l4_kern_desc_t) ((l4_word_t) kip + kip->kern_desc_ptr);
  if (kern->id.id == 0 && kern->id.subid == 1)
    printf ("L4/486");
  else if (kern->id.id == 0 && kern->id.subid == 2)
    printf ("L4/Pentium");
  else if (kern->id.id == 0 && kern->id.subid == 3)
    printf ("L4/x86");
  else if (kern->id.id == 1 && kern->id.subid == 1)
    printf ("L4/Mips");
  else if (kern->id.id == 2 && kern->id.subid == 1)
    printf ("L4/Alpha");
  else if (kern->id.id == 3 && kern->id.subid == 1)
    printf ("Fiasco");
  else if (kern->id.id == 4 && kern->id.subid == 1)
    printf ("L4Ka/Hazelnut");
  else if (kern->id.id == 4 && kern->id.subid == 2)
    printf ("L4Ka/Pistachio");
  else if (kern->id.id == 4 && kern->id.subid == 3)
    printf ("L4Ka/Strawberry");
  else
    printf ("Unknown Kernel");
  printf (" from ");

  if (kern->supplier[0] == 'G' && kern->supplier[1] == 'M'
      && kern->supplier[2] == 'D' && kern->supplier[3] == ' ')
    printf ("GMD");
  else if (kern->supplier[0] == 'I' && kern->supplier[1] == 'B'
      && kern->supplier[2] == 'M' && kern->supplier[3] == ' ')
    printf ("IBM Research");
  else if (kern->supplier[0] == 'U' && kern->supplier[1] == 'N'
      && kern->supplier[2] == 'S' && kern->supplier[3] == 'W')
    printf ("University of New South Wales, Sydney");
  else if (kern->supplier[0] == 'T' && kern->supplier[1] == 'U'
      && kern->supplier[2] == 'D' && kern->supplier[3] == ' ')
    printf ("Technische Universitaet Dresden");
  else if (kern->supplier[0] == 'U' && kern->supplier[1] == 'K'
      && kern->supplier[2] == 'a' && kern->supplier[3] == ' ')
    printf ("Universitaet Karlsruhe (TH)");
  else
    printf ("Unknown Supplier");
  printf (" (0x%x, 0x%x, `%c%c%c%c')\n",
	  kern->id.id, kern->id.subid, kern->supplier[0],
	  kern->supplier[1], kern->supplier[2], kern->supplier[3]);
  printf ("        Version %u.%u.%u, generated %u/%u/%u\n",
	  kern->version.ver, kern->version.subver, kern->version.subsubver,
	  kern->gen_date.year + 2000, kern->gen_date.month,
	  kern->gen_date.day);
  printf ("        %s\n", kern->version_parts);
  features = kern->version_parts;
  while (*features)
    features++;
  features++;
  if (*features)
    {
      printf ("        [");
      while (*features)
	{
	  printf ("%s", features);
	  while (*features)
	    features++;
	  features++;
	  if (*features)
	    printf (", ");
	}
      printf ("]\n");
    }

  nr = kip->processor_info.processors + 1;
  printf ("Processor: %u (", nr);
  proc = (l4_proc_desc_t) ((char *) kip + kip->proc_desc_ptr);
  while (nr--)
    {
      printf ("I %u kHz E %u kHz", proc->internal_freq, proc->external_freq);
      if (nr)
	printf (",");
      proc = (l4_proc_desc_t) ((l4_word_t) proc
			       + (1 << kip->processor_info.log2_size));
    }
  printf (")\n");

  printf ("Clock: Read precision %u, Schedule precision %u\n",
	  kip->clock_info.read_precision, kip->clock_info.schedule_precision);

  printf ("Page Info: Access %c%c%c, Sizes: ",
	  kip->page_info.read ? 'r' : '-',
	  kip->page_info.write ? 'w' : '-',
	  kip->page_info.execute ? 'x' : '-');
  for (nr = 0; nr < (sizeof (l4_word_t) * 8) - 10; nr++)
    {
      if (kip->page_info.page_size_mask & (1 << nr))
	printf ("%u ", nr);
    }
  printf ("\n");

  printf ("Threads: User Base %u, System Base %u, Thread No Bits %u\n",
	  kip->thread_info.user_base, kip->thread_info.system_base,
	  kip->thread_info.log2_max_thread);

  printf ("UTCB Info: Minimum Area Size 0x%x, Alignment Log2 %u, Size 0x%x\n",
	  1 << kip->utcb_info.log2_min_size,
	  kip->utcb_info.log2_align,
	  (1 << kip->utcb_info.log2_align) * kip->utcb_info.size_mul);

  nr = kip->memory_info.nr;
  printf ("Memory: %u\n", nr);
  mem = (l4_memory_desc_t) ((char *) kip + kip->memory_info.mem_desc_ptr);
  while (nr--)
    {
      printf ("        0x%x - 0x%x %s ",
	      mem->low, mem->high, mem->virtual ? "virtual " : "physical");
      if (mem->type == L4_MEMDESC_UNDEFINED)
	printf ("undefined");
      else if (mem->type == L4_MEMDESC_CONVENTIONAL)
	printf ("conventional");
      else if (mem->type == L4_MEMDESC_RESERVED)
	printf ("reserved");
      else if (mem->type == L4_MEMDESC_DEDICATED)
	printf ("dedicated");
      else if (mem->type == L4_MEMDESC_SHARED)
	printf ("shared");
      else if (mem->type == L4_MEMDESC_BOOTLOADER)
	printf ("boot loader 0x%x", mem->subtype);
      else if (mem->type == L4_MEMDESC_ARCH)
	printf ("architecture 0x%x", mem->subtype);
      else
	printf ("unknown");
      printf ("\n");
      mem++;
    }

  printf ("Boot Info: 0x%x\n", kip->boot_info);

  return 0;
}
